searchData={"items":[{"type":"module","title":"LiveFilter","doc":"LiveFilter is a comprehensive, reusable filtering library for Phoenix LiveView applications.\n\nInspired by modern tools like Notion and Airtable, it provides a complete solution for \nfiltering, sorting, and pagination with clean URL state management and a modern UI \nbuilt on SaladUI.","ref":"LiveFilter.html"},{"type":"module","title":"Key Features - LiveFilter","doc":"- **Complete Filter System**: 20+ operators across all data types (string, numeric, boolean, date, enum, array)\n- **Database Pagination**: Efficient queries that scale to any dataset size\n- **URL State Management**: Shareable, bookmarkable URLs with complete state restoration\n- **Table Sorting**: Multi-column sorting with visual indicators\n- **Column Management**: Dynamic column visibility\n- **Modern UI**: Built on SaladUI with shadcn/ui design system\n- **Production Ready**: Comprehensive validation, error handling, and performance optimization","ref":"LiveFilter.html#module-key-features"},{"type":"module","title":"Quick Start - LiveFilter","doc":"Add LiveFilter to your Phoenix LiveView:\n\n    defmodule MyAppWeb.ProductLive do\n      use MyAppWeb, :live_view\n      use LiveFilter.Mountable\n\n      def mount(_params, _session, socket) do\n        socket = mount_filters(socket, registry: my_field_registry())\n        {:ok, socket}\n      end\n\n      def handle_params(params, _url, socket) do\n        socket = handle_filter_params(socket, params)\n        {:noreply, socket}\n      end\n    end","ref":"LiveFilter.html#module-quick-start"},{"type":"module","title":"JavaScript Assets - LiveFilter","doc":"Some components require JavaScript hooks. Install them with:\n\n    mix live_filter.install.assets\n\nThen add to your app.js:\n\n    import LiveFilter from \"./hooks/live_filter/live_filter\"\n    \n    let liveSocket = new LiveSocket(\"/live\", Socket, {\n      hooks: { LiveFilter }\n    })","ref":"LiveFilter.html#module-javascript-assets"},{"type":"module","title":"Main Modules - LiveFilter","doc":"- `LiveFilter.Mountable` - LiveView integration\n- `LiveFilter.Components.*` - UI components\n- `LiveFilter.QueryBuilder` - Ecto query generation\n- `LiveFilter.UrlSerializer` - URL state management\n- `LiveFilter.Field` - Field type system","ref":"LiveFilter.html#module-main-modules"},{"type":"function","title":"LiveFilter.version/0","doc":"Returns the current version of LiveFilter.","ref":"LiveFilter.html#version/0"},{"type":"module","title":"LiveFilter.Components.ColumnSettings","doc":"A component for managing table column visibility.\nShows a gear icon that opens a dropdown with checkboxes for each column.","ref":"LiveFilter.Components.ColumnSettings.html"},{"type":"module","title":"Examples - LiveFilter.Components.ColumnSettings","doc":"<.live_component\n      module={LiveFilter.Components.ColumnSettings}\n      id=\"column-settings\"\n      columns={@column_config}\n      visible_columns={@visible_columns}\n    />","ref":"LiveFilter.Components.ColumnSettings.html#module-examples"},{"type":"module","title":"LiveFilter.Components.DateRangeSelect","doc":"A reusable date range selector component for LiveFilter.\nProvides preset date ranges and custom date range selection with a calendar picker.","ref":"LiveFilter.Components.DateRangeSelect.html"},{"type":"module","title":"JavaScript Hook Dependency - LiveFilter.Components.DateRangeSelect","doc":"This component requires the LiveFilter JavaScript hook to function properly.\nThe hook handles positioning of the calendar dropdown to prevent viewport overflow and\nmaintains stable positioning during date selection.","ref":"LiveFilter.Components.DateRangeSelect.html#module-javascript-hook-dependency"},{"type":"module","title":"Installation - LiveFilter.Components.DateRangeSelect","doc":"When using LiveFilter as a library, run the following to install JavaScript assets:\n\n    mix live_filter.install.assets\n\nThen import the hook in your `app.js`:\n\n    import LiveFilter from \"./hooks/live_filter/live_filter\"\n    \n    let liveSocket = new LiveSocket(\"/live\", Socket, {\n      hooks: { LiveFilter }\n    })","ref":"LiveFilter.Components.DateRangeSelect.html#module-installation"},{"type":"module","title":"Hook Details - LiveFilter.Components.DateRangeSelect","doc":"The LiveFilter hook automatically detects this component and:\n- Calculates optimal calendar position to stay within viewport\n- Handles window resize events\n- Maintains position stability during date selection\n- Uses `phx-hook=\"LiveFilter\"` with `data-livefilter-component=\"date-range-select\"`","ref":"LiveFilter.Components.DateRangeSelect.html#module-hook-details"},{"type":"module","title":"LiveFilter.Components.FilterBuilder","doc":"A component for building complex filter queries with a UI.\n\nAccepts:\n- filter_group: The FilterGroup struct\n- field_options: List of {field, label, type, opts} tuples\n- field_value_options: Map of field => options list for enum/array fields\n- ui_components: Optional UI component configuration","ref":"LiveFilter.Components.FilterBuilder.html"},{"type":"module","title":"LiveFilter.Components.FilterItem","doc":"A component for rendering individual filter items with configurable UI.\n\nAccepts:\n- filter: The filter struct\n- field_options: List of {field, label, type, opts} tuples\n- field_value_options: Map of field => options list for enum/array fields\n- ui_components: Optional UI component configuration","ref":"LiveFilter.Components.FilterItem.html"},{"type":"module","title":"LiveFilter.Components.FilterSelector","doc":"A dropdown component for selecting filters to add to the active filter set.\n\nThis component provides a clean UI for adding optional filters without taking up\ntoolbar space for filters that aren't actively in use.","ref":"LiveFilter.Components.FilterSelector.html"},{"type":"module","title":"Features - LiveFilter.Components.FilterSelector","doc":"- Shows only inactive filters in the dropdown\n- Supports custom icons for each filter\n- Sends standardized events when filters are selected\n- Fully optional - can be used alongside or instead of FilterBuilder","ref":"LiveFilter.Components.FilterSelector.html#module-features"},{"type":"module","title":"Usage - LiveFilter.Components.FilterSelector","doc":"<.live_component\n      module={LiveFilter.Components.FilterSelector}\n      id=\"add-filter\"\n      available_filters={[\n        {:description, \"Description\", \"hero-document-text\"},\n        {:tags, \"Tags\", \"hero-tag\"},\n        {:estimated_hours, \"Est. Hours\", \"hero-clock\"}\n      ]}\n      active_filters={[:tags]}\n      label=\"Add Filter\"\n      class=\"my-custom-class\"\n    />\n\nWhen a filter is selected, it sends a message to the parent LiveView:\n\n    {:filter_selected, filter_field}","ref":"LiveFilter.Components.FilterSelector.html#module-usage"},{"type":"module","title":"LiveFilter.Components.QuickFilter","doc":"A dynamic filter component that renders the appropriate input based on filter type.\n\nThis component automatically detects the filter type and renders the appropriate\nUI element (text input, select, date picker, etc.) with consistent styling and\nbehavior across all filter types.","ref":"LiveFilter.Components.QuickFilter.html"},{"type":"module","title":"Supported Types - LiveFilter.Components.QuickFilter","doc":"- `:string` - Text input\n- `:integer`, `:float` - Number input\n- `:boolean` - Toggle/checkbox\n- `:date` - Date picker\n- `:datetime` - DateTime picker\n- `:enum` - Single select dropdown\n- `:array` - Multi-select dropdown","ref":"LiveFilter.Components.QuickFilter.html#module-supported-types"},{"type":"module","title":"Usage - LiveFilter.Components.QuickFilter","doc":"<.live_component\n      module={LiveFilter.Components.QuickFilter}\n      id=\"filter-title\"\n      field={:title}\n      label=\"Title\"\n      type={:string}\n      value={@title_filter}\n      icon=\"hero-document\"\n    />\n\nSends messages to parent LiveView:\n- `{:quick_filter_changed, field, value}` when value changes\n- `{:quick_filter_cleared, field}` when filter is cleared","ref":"LiveFilter.Components.QuickFilter.html#module-usage"},{"type":"function","title":"LiveFilter.Components.QuickFilter.handle_info/2","doc":"","ref":"LiveFilter.Components.QuickFilter.html#handle_info/2"},{"type":"module","title":"LiveFilter.Components.SearchSelect","doc":"A search-enabled select component with single and multi-select support.\n\nFeatures:\n- In-dropdown search filtering\n- Single or multi-select modes\n- Configurable display of selected values\n- Item count badges\n- Clear all functionality\n- Keyboard navigation","ref":"LiveFilter.Components.SearchSelect.html"},{"type":"function","title":"LiveFilter.Components.SearchSelect.search_select/1","doc":"Renders a search-enabled select dropdown.","ref":"LiveFilter.Components.SearchSelect.html#search_select/1"},{"type":"function","title":"Options Format - LiveFilter.Components.SearchSelect.search_select/1","doc":"Options should be a list of tuples: `[{value, label}, ...]` or `[{value, label, count}, ...]`","ref":"LiveFilter.Components.SearchSelect.html#search_select/1-options-format"},{"type":"function","title":"Examples - LiveFilter.Components.SearchSelect.search_select/1","doc":"<.search_select\n      id=\"status-filter\"\n      options={[\n        {:pending, \"Pending\", 12},\n        {:in_progress, \"In Progress\", 8},\n        {:completed, \"Completed\", 45}\n      ]}\n      selected={[:pending, :in_progress]}\n      on_change=\"update_status_filter\"\n    />","ref":"LiveFilter.Components.SearchSelect.html#search_select/1-examples"},{"type":"function","title":"Attributes - LiveFilter.Components.SearchSelect.search_select/1","doc":"* `id` (`:string`) (required)\n* `options` (`:list`) (required)\n* `selected` (`:list`) - Defaults to `[]`.\n* `on_change` (`:any`) (required)\n* `multiple` (`:boolean`) - Defaults to `true`.\n* `placeholder` (`:string`) - Defaults to `\"Select...\"`.\n* `display_count` (`:integer`) - Defaults to `3`.\n* `searchable` (`:boolean`) - Defaults to `true`.\n* `clearable` (`:boolean`) - Defaults to `true`.\n* `label` (`:string`) - Defaults to `nil`.\n* `icon` (`:string`) - Defaults to `nil`.\n* `class` (`:string`) - Defaults to `nil`.\n* `size` (`:string`) - Defaults to `\"sm\"`. Must be one of `\"sm\"`, `\"md\"`, or `\"lg\"`.\n* `show_label_in_selection` (`:boolean`) - Defaults to `false`.\n* `clear_icon` (`:string`) - Defaults to `\"hero-x-circle\"`.\n* `plus_icon` (`:string`) - Defaults to `\"hero-plus-circle\"`.","ref":"LiveFilter.Components.SearchSelect.html#search_select/1-attributes"},{"type":"module","title":"LiveFilter.Components.Select","doc":"A simple dropdown select component for single selection.\n\nFeatures:\n- Single selection from options\n- Configurable display format\n- Optional icon and label\n- Follows LiveFilter component patterns","ref":"LiveFilter.Components.Select.html"},{"type":"function","title":"LiveFilter.Components.Select.select/1","doc":"Renders a simple select dropdown.","ref":"LiveFilter.Components.Select.html#select/1"},{"type":"function","title":"Options Format - LiveFilter.Components.Select.select/1","doc":"Options should be a list of tuples: `[{value, label}, ...]`","ref":"LiveFilter.Components.Select.html#select/1-options-format"},{"type":"function","title":"Examples - LiveFilter.Components.Select.select/1","doc":"<.select\n      id=\"sort-field\"\n      options={[\n        {\"title\", \"Title\"},\n        {\"status\", \"Status\"},\n        {\"created_at\", \"Created\"}\n      ]}\n      selected=\"title\"\n      on_change=\"update_sort\"\n      label=\"Sort\"\n    />","ref":"LiveFilter.Components.Select.html#select/1-examples"},{"type":"function","title":"Attributes - LiveFilter.Components.Select.select/1","doc":"* `id` (`:string`) (required)\n* `options` (`:list`) (required)\n* `selected` (`:string`) - Defaults to `nil`.\n* `on_change` (`:any`) (required)\n* `placeholder` (`:string`) - Defaults to `\"Select...\"`.\n* `label` (`:string`) - Defaults to `nil`.\n* `icon` (`:string`) - Defaults to `nil`.\n* `class` (`:string`) - Defaults to `nil`.\n* `size` (`:string`) - Defaults to `\"sm\"`. Must be one of `\"sm\"`, `\"md\"`, or `\"lg\"`.\n* `show_label_with_selection` (`:boolean`) - Defaults to `true`.\n* `clearable` (`:boolean`) - Defaults to `false`.","ref":"LiveFilter.Components.Select.html#select/1-attributes"},{"type":"module","title":"LiveFilter.Components.SortSelector","doc":"A component for selecting and managing sort options.\nCan be used in toolbars or dialogs for single or multi-field sorting.","ref":"LiveFilter.Components.SortSelector.html"},{"type":"module","title":"Examples - LiveFilter.Components.SortSelector","doc":"<.sort_selector\n      id=\"sort-selector\"\n      current_sorts={@current_sorts}\n      field_options={@field_options}\n      target={@myself}\n    />","ref":"LiveFilter.Components.SortSelector.html#module-examples"},{"type":"module","title":"LiveFilter.Components.SortableHeader","doc":"A sortable table header component that displays sort direction indicators\nand handles click events for sorting.","ref":"LiveFilter.Components.SortableHeader.html"},{"type":"module","title":"Examples - LiveFilter.Components.SortableHeader","doc":"<.sortable_header\n      field={:due_date}\n      label=\"Due Date\"\n      current_sort={@current_sort}\n      phx-click=\"sort_by\"\n    />","ref":"LiveFilter.Components.SortableHeader.html#module-examples"},{"type":"function","title":"LiveFilter.Components.SortableHeader.sortable_header/1","doc":"","ref":"LiveFilter.Components.SortableHeader.html#sortable_header/1"},{"type":"function","title":"Attributes - LiveFilter.Components.SortableHeader.sortable_header/1","doc":"* `field` (`:atom`) (required)\n* `label` (`:string`) (required)\n* `current_sort` (`:any`) - Defaults to `nil`.\n* `class` (`:string`) - Defaults to `\"\"`.\n* Global attributes are accepted. Supports all globals plus: `[\"phx-click\", \"phx-target\"]`.","ref":"LiveFilter.Components.SortableHeader.html#sortable_header/1-attributes"},{"type":"module","title":"LiveFilter.DateUtils","doc":"Date utility functions for LiveFilter.\nHandles date range calculations for presets and custom ranges.","ref":"LiveFilter.DateUtils.html"},{"type":"function","title":"LiveFilter.DateUtils.convert_range_to_type/2","doc":"Converts a date range to the specified timestamp type.","ref":"LiveFilter.DateUtils.html#convert_range_to_type/2"},{"type":"function","title":"LiveFilter.DateUtils.convert_to_type/3","doc":"Converts a single date/datetime to the specified type.\nFor ranges, use :start for beginning of day and :end for end of day.","ref":"LiveFilter.DateUtils.html#convert_to_type/3"},{"type":"function","title":"LiveFilter.DateUtils.parse_date_range/2","doc":"Converts a date preset string or date range tuple to a proper date range.\nSupports optional timestamp_type parameter for converting dates to timestamps.","ref":"LiveFilter.DateUtils.html#parse_date_range/2"},{"type":"module","title":"LiveFilter.EventRouter","doc":"Optional event routing helper for dynamic filter events.\n\nThis module helps parse and route events with dynamic names like\n\"filter_status_changed\" or \"quick_filter_project_selected\". It's\ncompletely optional - you can handle events however you prefer.","ref":"LiveFilter.EventRouter.html"},{"type":"module","title":"Philosophy - LiveFilter.EventRouter","doc":"This is a toolkit for common patterns, not a required way of handling events.\nUse it if it helps, ignore it if it doesn't fit your needs.","ref":"LiveFilter.EventRouter.html#module-philosophy"},{"type":"module","title":"Examples - LiveFilter.EventRouter","doc":"# In your LiveView\n    def handle_event(event, params, socket) do\n      case LiveFilter.EventRouter.parse_filter_event(event) do\n        {:ok, :status, :changed} ->\n          # Handle status filter change\n\n        {:ok, :date_range, :cleared} ->\n          # Handle date range clear\n\n        {:error, :no_match} ->\n          # Not a filter event, handle normally\n      end\n    end\n\n    # Or use the router function\n    def handle_event(event, params, socket) do\n      LiveFilter.EventRouter.route_event(event, params,\n        handlers: %{\n          status_changed: &handle_status_change/3,\n          date_range_selected: &handle_date_selection/3\n        },\n        fallback: &handle_other_event/3\n      )\n    end","ref":"LiveFilter.EventRouter.html#module-examples"},{"type":"function","title":"LiveFilter.EventRouter.build_event_name/3","doc":"Build a dynamic event name from components.\n\nThe inverse of parse_filter_event/2.","ref":"LiveFilter.EventRouter.html#build_event_name/3"},{"type":"function","title":"Examples - LiveFilter.EventRouter.build_event_name/3","doc":"build_event_name(:status, :changed)\n    #=> \"filter_status_changed\"\n\n    build_event_name(:due_date, :selected, prefix: \"quick_\")\n    #=> \"quick_due_date_selected\"","ref":"LiveFilter.EventRouter.html#build_event_name/3-examples"},{"type":"function","title":"LiveFilter.EventRouter.extract_event_value/2","doc":"Extract value from event parameters based on common patterns.","ref":"LiveFilter.EventRouter.html#extract_event_value/2"},{"type":"function","title":"Options - LiveFilter.EventRouter.extract_event_value/2","doc":"* `:type` - Expected value type (:single, :multi, :toggle, :range)\n  * `:key` - Parameter key to look for","ref":"LiveFilter.EventRouter.html#extract_event_value/2-options"},{"type":"function","title":"Examples - LiveFilter.EventRouter.extract_event_value/2","doc":"# Toggle event: %{\"toggle\" => \"active\"}\n    extract_event_value(%{\"toggle\" => \"active\"}, type: :toggle)\n    #=> {:toggle, \"active\"}\n\n    # Select event: %{\"select\" => \"urgent\"}\n    extract_event_value(%{\"select\" => \"urgent\"}, type: :single)\n    #=> {:select, \"urgent\"}\n\n    # Clear event: %{\"clear\" => true}\n    extract_event_value(%{\"clear\" => true})\n    #=> {:clear, true}","ref":"LiveFilter.EventRouter.html#extract_event_value/2-examples"},{"type":"function","title":"LiveFilter.EventRouter.filter_change_handler/1","doc":"Common event handler builder for filter changes.\n\nReturns a function that can be used as an event handler.","ref":"LiveFilter.EventRouter.html#filter_change_handler/1"},{"type":"function","title":"Examples - LiveFilter.EventRouter.filter_change_handler/1","doc":"def handle_event(event, params, socket) do\n      handler = LiveFilter.EventRouter.filter_change_handler(\n        field: :status,\n        type: :multi_select,\n        apply_fn: &apply_filters/1\n      )\n\n      handler.(params, socket)\n    end","ref":"LiveFilter.EventRouter.html#filter_change_handler/1-examples"},{"type":"function","title":"LiveFilter.EventRouter.parse_filter_event/2","doc":"Parse a dynamic event name into components.","ref":"LiveFilter.EventRouter.html#parse_filter_event/2"},{"type":"function","title":"Options - LiveFilter.EventRouter.parse_filter_event/2","doc":"* `:prefix` - Event prefix (default: \"filter_\")\n  * `:separator` - Word separator (default: \"_\")\n  * `:actions` - Recognized action suffixes (default: common actions)\n  * `:fields` - Whitelist of valid fields (default: any)","ref":"LiveFilter.EventRouter.html#parse_filter_event/2-options"},{"type":"function","title":"Examples - LiveFilter.EventRouter.parse_filter_event/2","doc":"# Default pattern: \"filter_ _ \"\n    parse_filter_event(\"filter_status_changed\")\n    #=> {:ok, :status, :changed}\n\n    parse_filter_event(\"filter_due_date_cleared\")\n    #=> {:ok, :due_date, :cleared}\n\n    # Custom pattern: \"quick_ _ \"\n    parse_filter_event(\"quick_project_selected\", prefix: \"quick_\")\n    #=> {:ok, :project, :selected}\n\n    # Unknown event\n    parse_filter_event(\"something_else\")\n    #=> {:error, :no_match}","ref":"LiveFilter.EventRouter.html#parse_filter_event/2-examples"},{"type":"function","title":"LiveFilter.EventRouter.route_event/3","doc":"Route events to appropriate handlers.\n\nThis is a higher-level helper that combines parsing with handler dispatch.","ref":"LiveFilter.EventRouter.html#route_event/3"},{"type":"function","title":"Options - LiveFilter.EventRouter.route_event/3","doc":"* `:handlers` - Map of \"field_action\" => handler function\n  * `:fallback` - Function to call for non-matching events\n  * `:parse_opts` - Options to pass to parse_filter_event/2","ref":"LiveFilter.EventRouter.html#route_event/3-options"},{"type":"function","title":"Examples - LiveFilter.EventRouter.route_event/3","doc":"LiveFilter.EventRouter.route_event(\"filter_status_changed\", params,\n      handlers: %{\n        \"status_changed\" => fn params, socket ->\n          # Handle status change\n          {:noreply, socket}\n        end,\n        \"date_range_selected\" => fn params, socket ->\n          # Handle date selection\n          {:noreply, socket}\n        end\n      },\n      fallback: fn event, params, socket ->\n        # Handle other events\n        {:noreply, socket}\n      end\n    )","ref":"LiveFilter.EventRouter.html#route_event/3-examples"},{"type":"protocol","title":"LiveFilter.Field","doc":"Protocol for defining custom field types in LiveFilter.\n\nThis protocol allows users to define completely custom field types with their own\nconversion logic, operators, and validation rules. It's entirely optional - users\ncan work with LiveFilter without ever implementing this protocol.","ref":"LiveFilter.Field.html"},{"type":"protocol","title":"Example Implementation - LiveFilter.Field","doc":"defmodule MyApp.PriorityField do\n      defstruct [:levels, :default]\n\n      def new(opts \\ []) do\n        %__MODULE__{\n          levels: Keyword.get(opts, :levels, [:low, :medium, :high, :urgent]),\n          default: Keyword.get(opts, :default, :medium)\n        }\n      end\n    end\n\n    defimpl LiveFilter.Field, for: MyApp.PriorityField do\n      def to_filter_value(%{levels: levels}, ui_value) do\n        # Convert UI selection to filter value\n        case ui_value do\n          \"urgent_only\" -> [:urgent]\n          \"high_and_up\" -> [:high, :urgent]\n          level when level in levels -> [level]\n          _ -> nil\n        end\n      end\n\n      def to_ui_value(_, filter_value) do\n        # Convert filter value back to UI representation\n        case filter_value do\n          [:urgent] -> \"urgent_only\"\n          [:high, :urgent] -> \"high_and_up\"\n          [level] -> level\n          _ -> nil\n        end\n      end\n\n      def default_operator(_), do: :in\n\n      def validate(%{levels: levels}, value) do\n        if Enum.all?(value, &(&1 in levels)) do\n          :ok\n        else\n          {:error, \"Invalid priority level\"}\n        end\n      end\n\n      def operators(_) do\n        [:in, :not_in, :equals]\n      end\n\n      def ui_component(_) do\n        :custom_priority_selector\n      end\n    end","ref":"LiveFilter.Field.html#module-example-implementation"},{"type":"function","title":"LiveFilter.Field.default_operator/1","doc":"Get the default operator for this field type.\n\nUsed when creating filters without an explicit operator.","ref":"LiveFilter.Field.html#default_operator/1"},{"type":"function","title":"Parameters - LiveFilter.Field.default_operator/1","doc":"* `field_type` - The field type struct","ref":"LiveFilter.Field.html#default_operator/1-parameters"},{"type":"function","title":"Returns - LiveFilter.Field.default_operator/1","doc":"An atom representing the default operator (e.g., `:equals`, `:contains`, `:in`)","ref":"LiveFilter.Field.html#default_operator/1-returns"},{"type":"function","title":"LiveFilter.Field.operators/1","doc":"Get available operators for this field type.","ref":"LiveFilter.Field.html#operators/1"},{"type":"function","title":"Parameters - LiveFilter.Field.operators/1","doc":"* `field_type` - The field type struct","ref":"LiveFilter.Field.html#operators/1-parameters"},{"type":"function","title":"Returns - LiveFilter.Field.operators/1","doc":"List of operator atoms that are valid for this field type.","ref":"LiveFilter.Field.html#operators/1-returns"},{"type":"type","title":"LiveFilter.Field.t/0","doc":"All the types that implement this protocol.","ref":"LiveFilter.Field.html#t:t/0"},{"type":"function","title":"LiveFilter.Field.to_filter_value/2","doc":"Convert a UI value (from form input, API, etc.) to a filter value.\n\nThis function handles the transformation from user input to the internal\nfilter representation.","ref":"LiveFilter.Field.html#to_filter_value/2"},{"type":"function","title":"Parameters - LiveFilter.Field.to_filter_value/2","doc":"* `field_type` - The field type struct\n  * `ui_value` - The value from the UI/form","ref":"LiveFilter.Field.html#to_filter_value/2-parameters"},{"type":"function","title":"Returns - LiveFilter.Field.to_filter_value/2","doc":"The transformed value suitable for use in a LiveFilter.Filter struct,\nor nil if the value should be ignored.","ref":"LiveFilter.Field.html#to_filter_value/2-returns"},{"type":"function","title":"LiveFilter.Field.to_ui_value/2","doc":"Convert a filter value back to UI representation.\n\nThis is the inverse of `to_filter_value/2` and is used when restoring\nUI state from URL parameters or saved filters.","ref":"LiveFilter.Field.html#to_ui_value/2"},{"type":"function","title":"Parameters - LiveFilter.Field.to_ui_value/2","doc":"* `field_type` - The field type struct\n  * `filter_value` - The value from a LiveFilter.Filter","ref":"LiveFilter.Field.html#to_ui_value/2-parameters"},{"type":"function","title":"Returns - LiveFilter.Field.to_ui_value/2","doc":"The value suitable for display/selection in the UI.","ref":"LiveFilter.Field.html#to_ui_value/2-returns"},{"type":"function","title":"LiveFilter.Field.ui_component/1","doc":"Get the suggested UI component for this field type.\n\nThis is just a hint - the actual UI implementation can use any component.","ref":"LiveFilter.Field.html#ui_component/1"},{"type":"function","title":"Parameters - LiveFilter.Field.ui_component/1","doc":"* `field_type` - The field type struct","ref":"LiveFilter.Field.html#ui_component/1-parameters"},{"type":"function","title":"Returns - LiveFilter.Field.ui_component/1","doc":"An atom identifying the suggested component type, or a map with component details.","ref":"LiveFilter.Field.html#ui_component/1-returns"},{"type":"function","title":"LiveFilter.Field.validate/2","doc":"Validate a value for this field type.","ref":"LiveFilter.Field.html#validate/2"},{"type":"function","title":"Parameters - LiveFilter.Field.validate/2","doc":"* `field_type` - The field type struct\n  * `value` - The value to validate","ref":"LiveFilter.Field.html#validate/2-parameters"},{"type":"function","title":"Returns - LiveFilter.Field.validate/2","doc":"* `:ok` if valid\n  * `{:error, reason}` if invalid","ref":"LiveFilter.Field.html#validate/2-returns"},{"type":"module","title":"LiveFilter.FieldRegistry","doc":"Standardized field configuration for LiveFilter.\n\nThe FieldRegistry provides a way to define and manage field configurations\nin a centralized way. It's completely optional - you can use LiveFilter\nwithout ever creating a registry.","ref":"LiveFilter.FieldRegistry.html"},{"type":"module","title":"Benefits - LiveFilter.FieldRegistry","doc":"* Centralized field definitions\n  * Type safety and validation\n  * Consistent operator defaults\n  * UI component hints\n  * Support for custom field types via the Field protocol","ref":"LiveFilter.FieldRegistry.html#module-benefits"},{"type":"module","title":"Example - LiveFilter.FieldRegistry","doc":"defmodule MyApp.FilterFields do\n      def registry do\n        LiveFilter.FieldRegistry.new()\n        |> LiveFilter.FieldRegistry.add_field(:title, :string,\n          label: \"Title\",\n          operators: [:contains, :equals, :starts_with],\n          placeholder: \"Search titles...\"\n        )\n        |> LiveFilter.FieldRegistry.add_field(:status, :enum,\n          label: \"Status\",\n          options: [\n            {\"Pending\", :pending},\n            {\"In Progress\", :in_progress},\n            {\"Completed\", :completed}\n          ],\n          default_operator: :in,\n          multiple: true\n        )\n        |> LiveFilter.FieldRegistry.add_field(:priority, MyApp.PriorityField.new(),\n          label: \"Priority\",\n          icon: \"hero-flag\"\n        )\n      end\n    end","ref":"LiveFilter.FieldRegistry.html#module-example"},{"type":"function","title":"LiveFilter.FieldRegistry.add_field/4","doc":"Add a field to the registry.","ref":"LiveFilter.FieldRegistry.html#add_field/4"},{"type":"function","title":"Parameters - LiveFilter.FieldRegistry.add_field/4","doc":"* `registry` - The registry to add to\n  * `name` - The field name (atom)\n  * `type` - Either an atom (:string, :integer, etc.) or a struct implementing Field protocol\n  * `opts` - Field configuration options","ref":"LiveFilter.FieldRegistry.html#add_field/4-parameters"},{"type":"function","title":"Options - LiveFilter.FieldRegistry.add_field/4","doc":"* `:label` - Human-readable label\n  * `:operators` - List of allowed operators (nil means use type defaults)\n  * `:default_operator` - Default operator (nil means use type default)\n  * `:required` - Whether the field is required\n  * `:placeholder` - UI placeholder text\n  * `:help_text` - Help text for users\n  * `:icon` - Icon identifier for UI\n  * `:group` - Group name for organizing fields\n  * `:options` - For enum/select fields, the available options\n  * `:multiple` - For enum fields, whether multiple selection is allowed\n  * `:validate` - Custom validation function\n  * Any other options are stored as-is","ref":"LiveFilter.FieldRegistry.html#add_field/4-options"},{"type":"function","title":"Examples - LiveFilter.FieldRegistry.add_field/4","doc":"# Simple field\n    registry\n    |> add_field(:title, :string, label: \"Title\")\n\n    # Enum field with options\n    registry\n    |> add_field(:status, :enum,\n      label: \"Status\",\n      options: [{\"Active\", :active}, {\"Inactive\", :inactive}],\n      default_operator: :in,\n      multiple: true\n    )\n\n    # Custom field type\n    registry\n    |> add_field(:priority, MyApp.PriorityField.new(), label: \"Priority\")","ref":"LiveFilter.FieldRegistry.html#add_field/4-examples"},{"type":"function","title":"LiveFilter.FieldRegistry.boolean_field/3","doc":"Helper to create a boolean field configuration.","ref":"LiveFilter.FieldRegistry.html#boolean_field/3"},{"type":"function","title":"LiveFilter.FieldRegistry.date_field/3","doc":"Helper to create a date field configuration.","ref":"LiveFilter.FieldRegistry.html#date_field/3"},{"type":"function","title":"LiveFilter.FieldRegistry.enum_field/4","doc":"Helper to create an enum field configuration.","ref":"LiveFilter.FieldRegistry.html#enum_field/4"},{"type":"type","title":"LiveFilter.FieldRegistry.field_config/0","doc":"","ref":"LiveFilter.FieldRegistry.html#t:field_config/0"},{"type":"function","title":"LiveFilter.FieldRegistry.from_fields/1","doc":"Create a registry from a list of field definitions.","ref":"LiveFilter.FieldRegistry.html#from_fields/1"},{"type":"function","title":"Example - LiveFilter.FieldRegistry.from_fields/1","doc":"fields = [\n      LiveFilter.FieldRegistry.string_field(:title, \"Title\"),\n      LiveFilter.FieldRegistry.enum_field(:status, \"Status\", [\n        {\"Active\", :active},\n        {\"Inactive\", :inactive}\n      ]),\n      {:custom, MyCustomField.new(), label: \"Custom Field\"}\n    ]\n\n    registry = LiveFilter.FieldRegistry.from_fields(fields)","ref":"LiveFilter.FieldRegistry.html#from_fields/1-example"},{"type":"function","title":"LiveFilter.FieldRegistry.get_default_operator/2","doc":"Get the default operator for a field.\n\nUses field config if specified, otherwise delegates to the Field protocol.","ref":"LiveFilter.FieldRegistry.html#get_default_operator/2"},{"type":"function","title":"LiveFilter.FieldRegistry.get_field/2","doc":"Get field configuration by name.\n\nReturns nil if field not found.","ref":"LiveFilter.FieldRegistry.html#get_field/2"},{"type":"function","title":"LiveFilter.FieldRegistry.get_fields_in_group/2","doc":"Get all fields in a group.","ref":"LiveFilter.FieldRegistry.html#get_fields_in_group/2"},{"type":"function","title":"LiveFilter.FieldRegistry.get_operators/2","doc":"Get available operators for a field.\n\nUses field config if specified, otherwise delegates to the Field protocol.","ref":"LiveFilter.FieldRegistry.html#get_operators/2"},{"type":"function","title":"LiveFilter.FieldRegistry.get_ui_component/2","doc":"Get the UI component suggestion for a field.","ref":"LiveFilter.FieldRegistry.html#get_ui_component/2"},{"type":"function","title":"LiveFilter.FieldRegistry.list_fields/1","doc":"List all fields in the registry.\n\nReturns a list of {name, config} tuples.","ref":"LiveFilter.FieldRegistry.html#list_fields/1"},{"type":"function","title":"LiveFilter.FieldRegistry.new/1","doc":"Create a new empty field registry.","ref":"LiveFilter.FieldRegistry.html#new/1"},{"type":"function","title":"Options - LiveFilter.FieldRegistry.new/1","doc":"* `:metadata` - Any metadata to attach to the registry","ref":"LiveFilter.FieldRegistry.html#new/1-options"},{"type":"function","title":"LiveFilter.FieldRegistry.string_field/3","doc":"Helper to create a string field configuration.","ref":"LiveFilter.FieldRegistry.html#string_field/3"},{"type":"type","title":"LiveFilter.FieldRegistry.t/0","doc":"","ref":"LiveFilter.FieldRegistry.html#t:t/0"},{"type":"function","title":"LiveFilter.FieldRegistry.to_filter_value/3","doc":"Convert UI value to filter value for a field.","ref":"LiveFilter.FieldRegistry.html#to_filter_value/3"},{"type":"function","title":"LiveFilter.FieldRegistry.to_ui_value/3","doc":"Convert filter value to UI value for a field.","ref":"LiveFilter.FieldRegistry.html#to_ui_value/3"},{"type":"function","title":"LiveFilter.FieldRegistry.validate/3","doc":"Validate a value for a field.\n\nUses custom validator if provided, otherwise delegates to Field protocol.","ref":"LiveFilter.FieldRegistry.html#validate/3"},{"type":"module","title":"LiveFilter.Filter","doc":"Represents an individual filter with field, operator, value, and type.\n\nA Filter defines a single filtering condition that can be applied to data queries.\nFilters are typically collected into FilterGroups for complex filtering logic.","ref":"LiveFilter.Filter.html"},{"type":"module","title":"Fields - LiveFilter.Filter","doc":"* `:field` - The field name to filter on (atom, e.g., `:title`, `:status`)\n  * `:operator` - The comparison operator (atom, e.g., `:equals`, `:contains`, `:greater_than`)\n  * `:value` - The value to compare against (any type, depends on operator and field type)\n  * `:type` - The field type for proper value handling (atom, e.g., `:string`, `:integer`, `:date`)","ref":"LiveFilter.Filter.html#module-fields"},{"type":"module","title":"Examples - LiveFilter.Filter","doc":"# String contains filter\n    %Filter{\n      field: :title,\n      operator: :contains,\n      value: \"elixir\",\n      type: :string\n    }\n    \n    # Integer comparison filter\n    %Filter{\n      field: :price,\n      operator: :greater_than,\n      value: 100,\n      type: :integer\n    }\n    \n    # Date range filter\n    %Filter{\n      field: :created_at,\n      operator: :between,\n      value: {~D[2024-01-01], ~D[2024-12-31]},\n      type: :date\n    }\n    \n    # Array membership filter\n    %Filter{\n      field: :tags,\n      operator: :contains_any,\n      value: [\"urgent\", \"bug\"],\n      type: :array\n    }\n    \n    # Boolean filter\n    %Filter{\n      field: :is_active,\n      operator: :is_true,\n      value: nil,  # Value ignored for boolean operators\n      type: :boolean\n    }","ref":"LiveFilter.Filter.html#module-examples"},{"type":"module","title":"Supported Operators by Type - LiveFilter.Filter","doc":"","ref":"LiveFilter.Filter.html#module-supported-operators-by-type"},{"type":"module","title":"String Types (:string, :text) - LiveFilter.Filter","doc":"- `:equals`, `:not_equals` - Exact matching\n- `:contains`, `:not_contains` - Substring search\n- `:starts_with`, `:ends_with` - Prefix/suffix matching\n- `:is_empty`, `:is_not_empty` - Null/empty checks","ref":"LiveFilter.Filter.html#module-string-types-string-text"},{"type":"module","title":"Numeric Types (:integer, :float) - LiveFilter.Filter","doc":"- `:equals`, `:not_equals` - Exact comparison\n- `:greater_than`, `:less_than` - Numeric comparison\n- `:greater_than_or_equal`, `:less_than_or_equal` - Inclusive comparison\n- `:between` - Range comparison (value should be `{min, max}` tuple)","ref":"LiveFilter.Filter.html#module-numeric-types-integer-float"},{"type":"module","title":"Date/Time Types (:date, :datetime, :utc_datetime) - LiveFilter.Filter","doc":"- `:equals`, `:not_equals` - Exact date comparison\n- `:before`, `:after` - Date comparison\n- `:on_or_before`, `:on_or_after` - Inclusive date comparison\n- `:between` - Date range (value should be `{start_date, end_date}` tuple)","ref":"LiveFilter.Filter.html#module-date-time-types-date-datetime-utc_datetime"},{"type":"module","title":"Boolean Type (:boolean) - LiveFilter.Filter","doc":"- `:is_true`, `:is_false` - Boolean value checks\n- `:is_empty`, `:is_not_empty` - Null checks","ref":"LiveFilter.Filter.html#module-boolean-type-boolean"},{"type":"module","title":"Array Types (:array, :multi_select) - LiveFilter.Filter","doc":"- `:contains_any` - Array overlap (PostgreSQL: `&&` operator)\n- `:contains_all` - Array contains (PostgreSQL: `@>` operator)\n- `:not_contains_any` - No array overlap\n- `:in`, `:not_in` - Membership checks","ref":"LiveFilter.Filter.html#module-array-types-array-multi_select"},{"type":"module","title":"Enum Types (:enum) - LiveFilter.Filter","doc":"- `:equals`, `:not_equals` - Exact matching\n- `:in`, `:not_in` - Multiple choice selection","ref":"LiveFilter.Filter.html#module-enum-types-enum"},{"type":"function","title":"LiveFilter.Filter.new/1","doc":"Creates a new filter struct.","ref":"LiveFilter.Filter.html#new/1"},{"type":"function","title":"Parameters - LiveFilter.Filter.new/1","doc":"* `attrs` - A map or keyword list of filter attributes","ref":"LiveFilter.Filter.html#new/1-parameters"},{"type":"function","title":"Examples - LiveFilter.Filter.new/1","doc":"iex> Filter.new(%{field: :title, operator: :contains, value: \"test\", type: :string})\n    %Filter{field: :title, operator: :contains, value: \"test\", type: :string}\n    \n    iex> Filter.new(field: :status, operator: :equals, value: \"active\", type: :enum)\n    %Filter{field: :status, operator: :equals, value: \"active\", type: :enum}","ref":"LiveFilter.Filter.html#new/1-examples"},{"type":"type","title":"LiveFilter.Filter.t/0","doc":"","ref":"LiveFilter.Filter.html#t:t/0"},{"type":"module","title":"LiveFilter.FilterGroup","doc":"Represents a group of filters with a conjunction (AND/OR) and can contain nested groups.","ref":"LiveFilter.FilterGroup.html"},{"type":"function","title":"LiveFilter.FilterGroup.add_filter/2","doc":"Adds a filter to the group.","ref":"LiveFilter.FilterGroup.html#add_filter/2"},{"type":"function","title":"Examples - LiveFilter.FilterGroup.add_filter/2","doc":"iex> group = %FilterGroup{}\n    iex> filter = %Filter{field: :title, operator: :contains, value: \"test\", type: :string}\n    iex> FilterGroup.add_filter(group, filter)\n    %FilterGroup{filters: [%Filter{field: :title, operator: :contains, value: \"test\", type: :string}]}","ref":"LiveFilter.FilterGroup.html#add_filter/2-examples"},{"type":"function","title":"LiveFilter.FilterGroup.add_group/2","doc":"Adds a nested group.","ref":"LiveFilter.FilterGroup.html#add_group/2"},{"type":"function","title":"Examples - LiveFilter.FilterGroup.add_group/2","doc":"iex> parent = %FilterGroup{conjunction: :and}\n    iex> child = %FilterGroup{conjunction: :or, filters: [%Filter{field: :status, operator: :equals, value: \"active\", type: :enum}]}\n    iex> FilterGroup.add_group(parent, child)\n    %FilterGroup{conjunction: :and, groups: [%FilterGroup{conjunction: :or, filters: [...]}]}","ref":"LiveFilter.FilterGroup.html#add_group/2-examples"},{"type":"function","title":"LiveFilter.FilterGroup.count_filters/1","doc":"Counts total filters including nested groups.","ref":"LiveFilter.FilterGroup.html#count_filters/1"},{"type":"function","title":"Examples - LiveFilter.FilterGroup.count_filters/1","doc":"iex> FilterGroup.count_filters(%FilterGroup{})\n    0\n    \n    iex> filter = %Filter{field: :title, operator: :contains, value: \"test\", type: :string}\n    iex> FilterGroup.count_filters(%FilterGroup{filters: [filter]})\n    1","ref":"LiveFilter.FilterGroup.html#count_filters/1-examples"},{"type":"function","title":"LiveFilter.FilterGroup.has_filters?/1","doc":"Checks if the group has any active filters.\n\nRecursively checks nested groups for filters.","ref":"LiveFilter.FilterGroup.html#has_filters?/1"},{"type":"function","title":"Examples - LiveFilter.FilterGroup.has_filters?/1","doc":"iex> FilterGroup.has_filters?(%FilterGroup{})\n    false\n    \n    iex> filter = %Filter{field: :title, operator: :contains, value: \"test\", type: :string}\n    iex> FilterGroup.has_filters?(%FilterGroup{filters: [filter]})\n    true","ref":"LiveFilter.FilterGroup.html#has_filters?/1-examples"},{"type":"function","title":"LiveFilter.FilterGroup.new/1","doc":"Creates a new filter group.","ref":"LiveFilter.FilterGroup.html#new/1"},{"type":"function","title":"Parameters - LiveFilter.FilterGroup.new/1","doc":"* `attrs` - A map or keyword list of filter group attributes","ref":"LiveFilter.FilterGroup.html#new/1-parameters"},{"type":"function","title":"Examples - LiveFilter.FilterGroup.new/1","doc":"iex> FilterGroup.new()\n    %FilterGroup{filters: [], groups: [], conjunction: :and}\n    \n    iex> FilterGroup.new(%{conjunction: :or})\n    %FilterGroup{filters: [], groups: [], conjunction: :or}","ref":"LiveFilter.FilterGroup.html#new/1-examples"},{"type":"function","title":"LiveFilter.FilterGroup.remove_filter/2","doc":"Removes a filter from the group by index.","ref":"LiveFilter.FilterGroup.html#remove_filter/2"},{"type":"function","title":"Examples - LiveFilter.FilterGroup.remove_filter/2","doc":"iex> filter = %Filter{field: :title, operator: :contains, value: \"test\", type: :string}\n    iex> group = %FilterGroup{filters: [filter]}\n    iex> FilterGroup.remove_filter(group, 0)\n    %FilterGroup{filters: []}","ref":"LiveFilter.FilterGroup.html#remove_filter/2-examples"},{"type":"type","title":"LiveFilter.FilterGroup.t/0","doc":"","ref":"LiveFilter.FilterGroup.html#t:t/0"},{"type":"function","title":"LiveFilter.FilterGroup.update_filter/3","doc":"Updates a filter in the group by index.","ref":"LiveFilter.FilterGroup.html#update_filter/3"},{"type":"function","title":"Examples - LiveFilter.FilterGroup.update_filter/3","doc":"iex> old_filter = %Filter{field: :title, operator: :contains, value: \"test\", type: :string}\n    iex> new_filter = %Filter{field: :title, operator: :equals, value: \"exact\", type: :string}\n    iex> group = %FilterGroup{filters: [old_filter]}\n    iex> FilterGroup.update_filter(group, 0, new_filter)\n    %FilterGroup{filters: [%Filter{field: :title, operator: :equals, value: \"exact\", type: :string}]}","ref":"LiveFilter.FilterGroup.html#update_filter/3-examples"},{"type":"module","title":"LiveFilter.FilterTypes","doc":"Defines available filter types and their operators.\nMaps field types to appropriate UI components.","ref":"LiveFilter.FilterTypes.html"},{"type":"function","title":"LiveFilter.FilterTypes.date_presets/0","doc":"Returns preset date ranges for date filters.","ref":"LiveFilter.FilterTypes.html#date_presets/0"},{"type":"function","title":"LiveFilter.FilterTypes.field_types/0","doc":"Returns all available field types.","ref":"LiveFilter.FilterTypes.html#field_types/0"},{"type":"function","title":"LiveFilter.FilterTypes.input_component_for/2","doc":"Maps a type and operator combination to an appropriate input component.","ref":"LiveFilter.FilterTypes.html#input_component_for/2"},{"type":"function","title":"LiveFilter.FilterTypes.operator_label/1","doc":"Returns human-readable label for an operator.","ref":"LiveFilter.FilterTypes.html#operator_label/1"},{"type":"function","title":"LiveFilter.FilterTypes.operator_requires_value?/1","doc":"Determines if an operator requires a value input.","ref":"LiveFilter.FilterTypes.html#operator_requires_value?/1"},{"type":"function","title":"LiveFilter.FilterTypes.operators_for_type/1","doc":"Returns available operators for a given field type.","ref":"LiveFilter.FilterTypes.html#operators_for_type/1"},{"type":"function","title":"LiveFilter.FilterTypes.valid_operator?/2","doc":"Validates if a given operator is valid for a field type.","ref":"LiveFilter.FilterTypes.html#valid_operator?/2"},{"type":"module","title":"LiveFilter.Mountable","doc":"Optional LiveView integration for LiveFilter.\n\nThis module provides a `use` macro that adds common filter-related functionality\nto your LiveView modules. Everything is overridable and customizable.","ref":"LiveFilter.Mountable.html"},{"type":"module","title":"Usage - LiveFilter.Mountable","doc":"defmodule MyAppWeb.ProductLive do\n      use MyAppWeb, :live_view\n      use LiveFilter.Mountable\n\n      def mount(params, session, socket) do\n        socket =\n          socket\n          |> mount_filters()  # Adds default filter assigns\n          |> assign(:products, [])\n\n        {:ok, socket}\n      end\n\n      def handle_params(params, _uri, socket) do\n        socket = handle_filter_params(socket, params)\n        {:noreply, load_products(socket)}\n      end\n    end","ref":"LiveFilter.Mountable.html#module-usage"},{"type":"module","title":"Customization - LiveFilter.Mountable","doc":"You can customize every aspect:\n\n    use LiveFilter.Mountable,\n      mount_assigns: [:custom_filter_state],\n      param_handler: &MyApp.custom_param_handler/3,\n      url_updater: &MyApp.custom_url_updater/2\n\nOr override the generated functions:\n\n    use LiveFilter.Mountable\n\n    # Override completely\n    def mount_filters(socket, opts) do\n      # Your custom implementation\n    end","ref":"LiveFilter.Mountable.html#module-customization"},{"type":"module","title":"LiveFilter.QueryBuilder","doc":"Converts filter configurations to Ecto queries using dynamic query building.\n\nThis module is the core of LiveFilter's database integration. It takes FilterGroup\nand Sort structures and converts them into efficient Ecto queries that can be \nexecuted against your database.","ref":"LiveFilter.QueryBuilder.html"},{"type":"module","title":"Features - LiveFilter.QueryBuilder","doc":"- **Dynamic Query Building**: Uses Ecto.Query.dynamic/2 for efficient query composition\n- **20+ Operators**: Supports comprehensive filtering operations for all data types\n- **Nested Logic**: Handles complex AND/OR combinations with FilterGroups\n- **PostgreSQL Arrays**: Special support for array operations (contains_any, contains_all)\n- **Null Handling**: Proper handling of nil values and empty checks\n- **Multi-column Sorting**: Supports single and multiple sort columns","ref":"LiveFilter.QueryBuilder.html#module-features"},{"type":"module","title":"Database Compatibility - LiveFilter.QueryBuilder","doc":"- **PostgreSQL**: Full support including array operations\n- **MySQL**: Supports all operators except array operations\n- **SQLite**: Supports all operators except array operations","ref":"LiveFilter.QueryBuilder.html#module-database-compatibility"},{"type":"module","title":"Performance Notes - LiveFilter.QueryBuilder","doc":"This module generates efficient SQL queries with proper WHERE clauses. For best\nperformance, ensure your database has appropriate indexes on filtered fields.","ref":"LiveFilter.QueryBuilder.html#module-performance-notes"},{"type":"module","title":"Examples - LiveFilter.QueryBuilder","doc":"# Simple filter\n    query = from p in Product\n    filter_group = %FilterGroup{\n      filters: [\n        %Filter{field: :name, operator: :contains, value: \"phone\", type: :string}\n      ]\n    }\n    QueryBuilder.build_query(query, filter_group)\n    \n    # Complex nested filters with sorting\n    filter_group = %FilterGroup{\n      filters: [\n        %Filter{field: :status, operator: :equals, value: \"active\", type: :enum}\n      ],\n      groups: [\n        %FilterGroup{\n          filters: [\n            %Filter{field: :price, operator: :greater_than, value: 100, type: :integer},\n            %Filter{field: :category, operator: :in, value: [\"electronics\", \"gadgets\"], type: :array}\n          ],\n          conjunction: :or\n        }\n      ],\n      conjunction: :and\n    }\n    \n    sorts = [\n      %Sort{field: :priority, direction: :desc},\n      %Sort{field: :created_at, direction: :asc}\n    ]\n    \n    query\n    |> QueryBuilder.build_query(filter_group)\n    |> QueryBuilder.apply_sort(sorts)","ref":"LiveFilter.QueryBuilder.html#module-examples"},{"type":"function","title":"LiveFilter.QueryBuilder.apply_sort/2","doc":"Applies sorting to a query.\n\nAccepts a single Sort struct, a list of Sort structs, or nil. When multiple\nsorts are provided, they are applied in order, creating a multi-column sort.","ref":"LiveFilter.QueryBuilder.html#apply_sort/2"},{"type":"function","title":"Parameters - LiveFilter.QueryBuilder.apply_sort/2","doc":"* `query` - The Ecto query to add sorting to\n  * `sorts` - A Sort struct, list of Sort structs, or nil","ref":"LiveFilter.QueryBuilder.html#apply_sort/2-parameters"},{"type":"function","title":"Returns - LiveFilter.QueryBuilder.apply_sort/2","doc":"The query with ORDER BY clauses applied, or the original query if no sorts.","ref":"LiveFilter.QueryBuilder.html#apply_sort/2-returns"},{"type":"function","title":"Examples - LiveFilter.QueryBuilder.apply_sort/2","doc":"# Single sort\n    apply_sort(query, %Sort{field: :due_date, direction: :asc})\n    \n    # Multiple sorts (applied in order - priority desc, then due_date asc)\n    apply_sort(query, [\n      %Sort{field: :priority, direction: :desc},\n      %Sort{field: :due_date, direction: :asc}\n    ])\n    \n    # No sorting\n    apply_sort(query, nil)  # Returns original query unchanged","ref":"LiveFilter.QueryBuilder.html#apply_sort/2-examples"},{"type":"function","title":"LiveFilter.QueryBuilder.build_query/2","doc":"Builds an Ecto query from a filter group.\n\nConverts a FilterGroup into dynamic Ecto query conditions using Ecto.Query.dynamic/2.\nReturns the original query unchanged if no filters are present.","ref":"LiveFilter.QueryBuilder.html#build_query/2"},{"type":"function","title":"Parameters - LiveFilter.QueryBuilder.build_query/2","doc":"* `query` - The base Ecto query to add conditions to\n  * `filter_group` - A FilterGroup struct containing filters and nested groups","ref":"LiveFilter.QueryBuilder.html#build_query/2-parameters"},{"type":"function","title":"Returns - LiveFilter.QueryBuilder.build_query/2","doc":"The query with WHERE conditions applied, or the original query if no filters.","ref":"LiveFilter.QueryBuilder.html#build_query/2-returns"},{"type":"function","title":"Examples - LiveFilter.QueryBuilder.build_query/2","doc":"iex> query = from p in Product\n    iex> filter_group = %FilterGroup{\n    ...>   filters: [\n    ...>     %Filter{field: :name, operator: :contains, value: \"phone\", type: :string}\n    ...>   ]\n    ...> }\n    iex> QueryBuilder.build_query(query, filter_group)\n    #Ecto.Query \n    \n    iex> empty_group = %FilterGroup{filters: []}\n    iex> QueryBuilder.build_query(query, empty_group) == query\n    true","ref":"LiveFilter.QueryBuilder.html#build_query/2-examples"},{"type":"module","title":"LiveFilter.QuickFilters","doc":"Composable filter builder functions for common patterns.\n\nThis module provides simple, focused functions for creating filters.\nEach function returns a Filter struct that can be used individually\nor combined with others. There's no magic - just convenience functions\nthat save you from writing boilerplate.","ref":"LiveFilter.QuickFilters.html"},{"type":"module","title":"Philosophy - LiveFilter.QuickFilters","doc":"These are building blocks, not a framework. Use what you need,\nignore what you don't, and write your own when these don't fit.","ref":"LiveFilter.QuickFilters.html#module-philosophy"},{"type":"module","title":"Examples - LiveFilter.QuickFilters","doc":"# Use individual builders\n    search = QuickFilters.search_filter(\"user input\")\n    status = QuickFilters.multi_select_filter(:status, [:active, :pending])\n    date = QuickFilters.date_range_filter(:created_at, {~D[2025-01-01], ~D[2025-01-31]})\n    \n    # Combine into a filter group\n    filter_group = %FilterGroup{\n      filters: [search, status, date] |> Enum.reject(&is_nil/1)\n    }\n    \n    # Or use in your own functions\n    def build_my_filters(params) do\n      [\n        params[\"q\"] && QuickFilters.search_filter(params[\"q\"]),\n        params[\"urgent\"] == \"true\" && QuickFilters.boolean_filter(:is_urgent, true),\n        # ... your custom logic\n      ]\n      |> Enum.reject(&is_nil/1)\n    end","ref":"LiveFilter.QuickFilters.html#module-examples"},{"type":"function","title":"LiveFilter.QuickFilters.array_filter/3","doc":"Create an array/tags filter.","ref":"LiveFilter.QuickFilters.html#array_filter/3"},{"type":"function","title":"Options - LiveFilter.QuickFilters.array_filter/3","doc":"* `:operator` - Operator (default: `:contains_any`)\n  * `:type` - Field type (default: `:array`)","ref":"LiveFilter.QuickFilters.html#array_filter/3-options"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.array_filter/3","doc":"# Contains any\n    QuickFilters.array_filter(:tags, [\"urgent\", \"bug\"])\n    \n    # Contains all\n    QuickFilters.array_filter(:tags, [\"urgent\", \"bug\"], operator: :contains_all)","ref":"LiveFilter.QuickFilters.html#array_filter/3-examples"},{"type":"function","title":"LiveFilter.QuickFilters.boolean_filter/3","doc":"Create a boolean filter.","ref":"LiveFilter.QuickFilters.html#boolean_filter/3"},{"type":"function","title":"Options - LiveFilter.QuickFilters.boolean_filter/3","doc":"* `:operator` - Operator to use (default: `:equals`)\n  * `:true_only` - Only create filter for true values (default: false)","ref":"LiveFilter.QuickFilters.html#boolean_filter/3-options"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.boolean_filter/3","doc":"# Simple boolean\n    QuickFilters.boolean_filter(:is_active, true)\n    \n    # Only filter when true\n    QuickFilters.boolean_filter(:is_urgent, value, true_only: true)","ref":"LiveFilter.QuickFilters.html#boolean_filter/3-examples"},{"type":"function","title":"LiveFilter.QuickFilters.date_range_filter/3","doc":"Create a date range filter.\n\nHandles both date tuples and preset strings.","ref":"LiveFilter.QuickFilters.html#date_range_filter/3"},{"type":"function","title":"Options - LiveFilter.QuickFilters.date_range_filter/3","doc":"* `:operator` - Operator to use (default: `:between` for ranges, `:equals` for single dates)\n  * `:type` - Field type (default: `:date`)\n  * `:timezone` - Timezone for date calculations (default: \"Etc/UTC\")","ref":"LiveFilter.QuickFilters.html#date_range_filter/3-options"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.date_range_filter/3","doc":"# Date range\n    QuickFilters.date_range_filter(:created_at, {~D[2025-01-01], ~D[2025-01-31]})\n    \n    # Single date\n    QuickFilters.date_range_filter(:due_date, ~D[2025-01-15])\n    \n    # Preset (requires LiveFilter.DateUtils)\n    QuickFilters.date_range_filter(:created_at, \"last_30_days\")","ref":"LiveFilter.QuickFilters.html#date_range_filter/3-examples"},{"type":"function","title":"LiveFilter.QuickFilters.example_multi_select_handler/4","doc":"Example handler for multi-select toggle.\n\nThis is just an example pattern.","ref":"LiveFilter.QuickFilters.html#example_multi_select_handler/4"},{"type":"function","title":"LiveFilter.QuickFilters.example_search_handler/3","doc":"Example event handler for search with debounce.\n\nThis is just an example - implement your own based on your needs.","ref":"LiveFilter.QuickFilters.html#example_search_handler/3"},{"type":"function","title":"LiveFilter.QuickFilters.extract_all/3","doc":"Extract multiple filter values at once.","ref":"LiveFilter.QuickFilters.html#extract_all/3"},{"type":"function","title":"Options - LiveFilter.QuickFilters.extract_all/3","doc":"* `:socket` - If provided, applies values to socket assigns","ref":"LiveFilter.QuickFilters.html#extract_all/3-options"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.extract_all/3","doc":"extractors = [\n      search_query: &extract_search_query/1,\n      selected_statuses: fn fg -> extract_multi_select(fg, :status) end,\n      is_urgent: fn fg -> extract_boolean(fg, :is_urgent, default: false) end\n    ]\n    \n    result = QuickFilters.extract_all(filter_group, extractors)\n    #=> %{search_query: \"test\", selected_statuses: [:active], is_urgent: false}\n    \n    # With socket\n    socket = QuickFilters.extract_all(filter_group, extractors, socket: socket)","ref":"LiveFilter.QuickFilters.html#extract_all/3-examples"},{"type":"function","title":"LiveFilter.QuickFilters.extract_array/3","doc":"Extract array value from a filter group.","ref":"LiveFilter.QuickFilters.html#extract_array/3"},{"type":"function","title":"Options - LiveFilter.QuickFilters.extract_array/3","doc":"* `:default` - Default value if not found (default: [])","ref":"LiveFilter.QuickFilters.html#extract_array/3-options"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.extract_array/3","doc":"filter_group = %FilterGroup{\n      filters: [\n        %Filter{field: :tags, operator: :contains_any, value: [\"urgent\", \"bug\"]}\n      ]\n    }\n    \n    QuickFilters.extract_array(filter_group, :tags)\n    #=> [\"urgent\", \"bug\"]","ref":"LiveFilter.QuickFilters.html#extract_array/3-examples"},{"type":"function","title":"LiveFilter.QuickFilters.extract_boolean/3","doc":"Extract boolean value from a filter group.","ref":"LiveFilter.QuickFilters.html#extract_boolean/3"},{"type":"function","title":"Options - LiveFilter.QuickFilters.extract_boolean/3","doc":"* `:default` - Default value if not found (default: nil)","ref":"LiveFilter.QuickFilters.html#extract_boolean/3-options"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.extract_boolean/3","doc":"filter_group = %FilterGroup{\n      filters: [\n        %Filter{field: :is_urgent, operator: :equals, value: true}\n      ]\n    }\n    \n    QuickFilters.extract_boolean(filter_group, :is_urgent)\n    #=> true","ref":"LiveFilter.QuickFilters.html#extract_boolean/3-examples"},{"type":"function","title":"LiveFilter.QuickFilters.extract_date_range/2","doc":"Extract date range from a filter group.\n\nReturns the date value or range tuple.","ref":"LiveFilter.QuickFilters.html#extract_date_range/2"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.extract_date_range/2","doc":"filter_group = %FilterGroup{\n      filters: [\n        %Filter{field: :due_date, operator: :between, value: {~D[2025-01-01], ~D[2025-01-31]}}\n      ]\n    }\n    \n    QuickFilters.extract_date_range(filter_group, :due_date)\n    #=> {~D[2025-01-01], ~D[2025-01-31]}","ref":"LiveFilter.QuickFilters.html#extract_date_range/2-examples"},{"type":"function","title":"LiveFilter.QuickFilters.extract_multi_select/3","doc":"Extract multi-select values from a filter group.\n\nReturns a list of selected values for enum/select fields.","ref":"LiveFilter.QuickFilters.html#extract_multi_select/3"},{"type":"function","title":"Options - LiveFilter.QuickFilters.extract_multi_select/3","doc":"* `:single_value` - Return single value instead of list for :equals (default: false)","ref":"LiveFilter.QuickFilters.html#extract_multi_select/3-options"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.extract_multi_select/3","doc":"filter_group = %FilterGroup{\n      filters: [\n        %Filter{field: :status, operator: :in, value: [:active, :pending]}\n      ]\n    }\n    \n    QuickFilters.extract_multi_select(filter_group, :status)\n    #=> [:active, :pending]","ref":"LiveFilter.QuickFilters.html#extract_multi_select/3-examples"},{"type":"function","title":"LiveFilter.QuickFilters.extract_numeric/2","doc":"Extract numeric value from a filter group.\n\nReturns the numeric value or range.","ref":"LiveFilter.QuickFilters.html#extract_numeric/2"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.extract_numeric/2","doc":"filter_group = %FilterGroup{\n      filters: [\n        %Filter{field: :price, operator: :greater_than, value: 99.99}\n      ]\n    }\n    \n    QuickFilters.extract_numeric(filter_group, :price)\n    #=> 99.99","ref":"LiveFilter.QuickFilters.html#extract_numeric/2-examples"},{"type":"function","title":"LiveFilter.QuickFilters.extract_optional_filters/3","doc":"Extract optional filters (filters not in exclusion list).\n\nReturns a map with `:active_optional_filters` and `:optional_filter_values`.","ref":"LiveFilter.QuickFilters.html#extract_optional_filters/3"},{"type":"function","title":"Options - LiveFilter.QuickFilters.extract_optional_filters/3","doc":"* `:socket` - If provided, applies values to socket assigns","ref":"LiveFilter.QuickFilters.html#extract_optional_filters/3-options"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.extract_optional_filters/3","doc":"excluded = [:status, :_search]\n    result = QuickFilters.extract_optional_filters(filter_group, excluded)\n    #=> %{\n    #=>   active_optional_filters: [:project, :tags],\n    #=>   optional_filter_values: %{project: \"phoenix\", tags: [\"bug\"]}\n    #=> }","ref":"LiveFilter.QuickFilters.html#extract_optional_filters/3-examples"},{"type":"function","title":"LiveFilter.QuickFilters.extract_search_query/2","doc":"Extract search query from a filter group.\n\nLooks for search filters and returns the query value.","ref":"LiveFilter.QuickFilters.html#extract_search_query/2"},{"type":"function","title":"Options - LiveFilter.QuickFilters.extract_search_query/2","doc":"* `:field` - Field to look for (default: `:_search`)\n  * `:operator` - Operator to match (default: any)","ref":"LiveFilter.QuickFilters.html#extract_search_query/2-options"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.extract_search_query/2","doc":"filter_group = %FilterGroup{\n      filters: [\n        %Filter{field: :_search, operator: :custom, value: \"search term\"}\n      ]\n    }\n    \n    QuickFilters.extract_search_query(filter_group)\n    #=> \"search term\"","ref":"LiveFilter.QuickFilters.html#extract_search_query/2-examples"},{"type":"function","title":"LiveFilter.QuickFilters.from_params/2","doc":"Build multiple filters from a parameter map.\n\nThis is a convenience function for handling common parameter patterns.","ref":"LiveFilter.QuickFilters.html#from_params/2"},{"type":"function","title":"Options - LiveFilter.QuickFilters.from_params/2","doc":"* `:definitions` - List of {param_key, builder_fun} tuples\n  * `:prefix` - Parameter key prefix to strip","ref":"LiveFilter.QuickFilters.html#from_params/2-options"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.from_params/2","doc":"params = %{\n      \"q\" => \"search term\",\n      \"status\" => [\"active\", \"pending\"],\n      \"urgent\" => \"true\"\n    }\n    \n    filters = QuickFilters.from_params(params,\n      definitions: [\n        {\"q\", fn v -> search_filter(v) end},\n        {\"status\", fn v -> multi_select_filter(:status, v) end},\n        {\"urgent\", fn \"true\" -> boolean_filter(:urgent, true); _ -> nil end}\n      ]\n    )","ref":"LiveFilter.QuickFilters.html#from_params/2-examples"},{"type":"function","title":"LiveFilter.QuickFilters.multi_select_filter/3","doc":"Create a multi-select filter for enum fields.","ref":"LiveFilter.QuickFilters.html#multi_select_filter/3"},{"type":"function","title":"Options - LiveFilter.QuickFilters.multi_select_filter/3","doc":"* `:operator` - Operator to use (default: `:in` for lists, `:equals` for single values)\n  * `:type` - Field type (default: `:enum`)\n  * `:reject_empty` - Whether to return nil for empty values (default: true)","ref":"LiveFilter.QuickFilters.html#multi_select_filter/3-options"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.multi_select_filter/3","doc":"# Multiple values\n    QuickFilters.multi_select_filter(:status, [:active, :pending])\n    \n    # Single value (uses :equals operator)\n    QuickFilters.multi_select_filter(:status, :active)\n    \n    # Force operator\n    QuickFilters.multi_select_filter(:tags, [\"urgent\", \"bug\"], operator: :contains_all)","ref":"LiveFilter.QuickFilters.html#multi_select_filter/3-examples"},{"type":"function","title":"LiveFilter.QuickFilters.numeric_filter/3","doc":"Create a numeric filter.","ref":"LiveFilter.QuickFilters.html#numeric_filter/3"},{"type":"function","title":"Options - LiveFilter.QuickFilters.numeric_filter/3","doc":"* `:operator` - Operator (default: `:equals`)\n  * `:type` - `:integer` or `:float` (default: auto-detect)","ref":"LiveFilter.QuickFilters.html#numeric_filter/3-options"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.numeric_filter/3","doc":"# Exact value\n    QuickFilters.numeric_filter(:price, 99.99)\n    \n    # Comparison\n    QuickFilters.numeric_filter(:age, 21, operator: :greater_than_or_equal)\n    \n    # Range\n    QuickFilters.numeric_filter(:score, {80, 100}, operator: :between)","ref":"LiveFilter.QuickFilters.html#numeric_filter/3-examples"},{"type":"function","title":"LiveFilter.QuickFilters.search_filter/2","doc":"Create a search filter.\n\nBy default creates a virtual `_search` filter that your application\ncan expand to search multiple fields. You can customize everything.","ref":"LiveFilter.QuickFilters.html#search_filter/2"},{"type":"function","title":"Options - LiveFilter.QuickFilters.search_filter/2","doc":"* `:field` - Field name (default: `:_search`)\n  * `:operator` - Operator to use (default: `:custom`)\n  * `:type` - Field type (default: `:string`)\n  * `:min_length` - Minimum query length to create filter (default: 0)\n  * `:trim` - Whether to trim whitespace (default: true)","ref":"LiveFilter.QuickFilters.html#search_filter/2-options"},{"type":"function","title":"Examples - LiveFilter.QuickFilters.search_filter/2","doc":"# Simple search\n    QuickFilters.search_filter(\"user query\")\n    \n    # Search specific field\n    QuickFilters.search_filter(\"john\", field: :author_name, operator: :contains)\n    \n    # Require minimum length\n    QuickFilters.search_filter(query, min_length: 3)\n\nReturns nil if query is empty or below min_length.","ref":"LiveFilter.QuickFilters.html#search_filter/2-examples"},{"type":"module","title":"LiveFilter.Sort","doc":"Represents a sort configuration for queries.\n\nA sort consists of:\n- field: The field to sort by (atom or string)\n- direction: :asc or :desc\n\nMultiple sorts can be applied in order (primary, secondary, etc.)","ref":"LiveFilter.Sort.html"},{"type":"type","title":"LiveFilter.Sort.direction/0","doc":"","ref":"LiveFilter.Sort.html#t:direction/0"},{"type":"function","title":"LiveFilter.Sort.new/2","doc":"Creates a new sort configuration.","ref":"LiveFilter.Sort.html#new/2"},{"type":"function","title":"Examples - LiveFilter.Sort.new/2","doc":"iex> LiveFilter.Sort.new(:due_date, :desc)\n    %LiveFilter.Sort{field: :due_date, direction: :desc}\n\n    iex> LiveFilter.Sort.new(\"created_at\")\n    %LiveFilter.Sort{field: \"created_at\", direction: :asc}","ref":"LiveFilter.Sort.html#new/2-examples"},{"type":"function","title":"LiveFilter.Sort.opposite_direction/1","doc":"Returns the opposite direction.","ref":"LiveFilter.Sort.html#opposite_direction/1"},{"type":"type","title":"LiveFilter.Sort.t/0","doc":"","ref":"LiveFilter.Sort.html#t:t/0"},{"type":"function","title":"LiveFilter.Sort.toggle_direction/1","doc":"Toggles the direction of a sort.","ref":"LiveFilter.Sort.html#toggle_direction/1"},{"type":"function","title":"Examples - LiveFilter.Sort.toggle_direction/1","doc":"iex> sort = LiveFilter.Sort.new(:title, :asc)\n    iex> LiveFilter.Sort.toggle_direction(sort)\n    %LiveFilter.Sort{field: :title, direction: :desc}","ref":"LiveFilter.Sort.html#toggle_direction/1-examples"},{"type":"module","title":"LiveFilter.TypeUtils","doc":"Utility functions for type conversion in LiveFilter.\n\nProvides safe conversion utilities for values that come from\nURL parameters or other external sources.","ref":"LiveFilter.TypeUtils.html"},{"type":"function","title":"LiveFilter.TypeUtils.convert_to_type/2","doc":"Safely converts a value to the specified type.\n\nSupports common type conversions needed by LiveFilter:\n- `:atom` - Convert strings to atoms\n- `:string` - Convert to string\n- `:integer` - Parse integers\n- `:float` - Parse floats\n- `:boolean` - Parse boolean values\n\nReturns `{:ok, converted_value}` or `{:error, reason}`.","ref":"LiveFilter.TypeUtils.html#convert_to_type/2"},{"type":"function","title":"Examples - LiveFilter.TypeUtils.convert_to_type/2","doc":"iex> LiveFilter.TypeUtils.convert_to_type(\"pending\", :atom)\n    {:ok, :pending}\n\n    iex> LiveFilter.TypeUtils.convert_to_type(\"123\", :integer)\n    {:ok, 123}\n\n    iex> LiveFilter.TypeUtils.convert_to_type(\"true\", :boolean)\n    {:ok, true}","ref":"LiveFilter.TypeUtils.html#convert_to_type/2-examples"},{"type":"function","title":"LiveFilter.TypeUtils.safe_to_atom/1","doc":"Converts string values to atoms safely, preserving non-string values.\n\nUsed for enum field values that come from URL parameters as strings\nbut need to be atoms for Ecto queries.","ref":"LiveFilter.TypeUtils.html#safe_to_atom/1"},{"type":"function","title":"Examples - LiveFilter.TypeUtils.safe_to_atom/1","doc":"iex> LiveFilter.TypeUtils.safe_to_atom(\"pending\")\n    :pending\n\n    iex> LiveFilter.TypeUtils.safe_to_atom(:already_atom)\n    :already_atom\n\n    iex> LiveFilter.TypeUtils.safe_to_atom([\"pending\", \"active\"])\n    [:pending, :active]","ref":"LiveFilter.TypeUtils.html#safe_to_atom/1-examples"},{"type":"function","title":"LiveFilter.TypeUtils.safe_to_string/1","doc":"Converts values to string safely.\n\nHandles atoms, strings, and other types gracefully.","ref":"LiveFilter.TypeUtils.html#safe_to_string/1"},{"type":"function","title":"Examples - LiveFilter.TypeUtils.safe_to_string/1","doc":"iex> LiveFilter.TypeUtils.safe_to_string(:pending)\n    \"pending\"\n\n    iex> LiveFilter.TypeUtils.safe_to_string(\"already_string\")\n    \"already_string\"\n\n    iex> LiveFilter.TypeUtils.safe_to_string(123)\n    \"123\"","ref":"LiveFilter.TypeUtils.html#safe_to_string/1-examples"},{"type":"module","title":"LiveFilter.UIState","doc":"Optional UI state management for LiveFilter.\n\nProvides helpers for managing UI state that maps to FilterGroup structures.\nAll functions can be overridden or ignored entirely - this module is purely\noptional and designed to reduce boilerplate for common patterns.","ref":"LiveFilter.UIState.html"},{"type":"module","title":"Usage - LiveFilter.UIState","doc":"# Use default converter\n    filters = LiveFilter.UIState.ui_to_filters(ui_state)\n\n    # Use custom converter\n    filters = LiveFilter.UIState.ui_to_filters(ui_state,\n      converter: &MyApp.custom_converter/2,\n      fields: MyApp.field_config()\n    )\n\n    # Extract values from filters back to UI\n    search_query = LiveFilter.UIState.extract_filter_value(filters, :_search)\n    selected_statuses = LiveFilter.UIState.extract_filter_value(filters, :status,\n      operator: :in,\n      default: []\n    )","ref":"LiveFilter.UIState.html#module-usage"},{"type":"function","title":"LiveFilter.UIState.__struct__/0","doc":"Suggested UI state structure. Users can use any map structure they prefer.","ref":"LiveFilter.UIState.html#__struct__/0"},{"type":"type","title":"LiveFilter.UIState.converter_fun/0","doc":"","ref":"LiveFilter.UIState.html#t:converter_fun/0"},{"type":"function","title":"LiveFilter.UIState.extract_filter_value/3","doc":"Extract a filter value from a list of filters or FilterGroup.","ref":"LiveFilter.UIState.html#extract_filter_value/3"},{"type":"function","title":"Options - LiveFilter.UIState.extract_filter_value/3","doc":"* `:extractor` - Custom extractor function\n  * `:operator` - Match specific operator (default: any)\n  * `:default` - Default value if not found\n  * `:transform` - Function to transform the extracted value","ref":"LiveFilter.UIState.html#extract_filter_value/3-options"},{"type":"function","title":"Examples - LiveFilter.UIState.extract_filter_value/3","doc":"# Extract search query\n    query = extract_filter_value(filters, :_search)\n\n    # Extract multi-select with specific operator\n    statuses = extract_filter_value(filters, :status,\n      operator: :in,\n      default: []\n    )\n\n    # Transform extracted value\n    assignee_names = extract_filter_value(filters, :assigned_to,\n      transform: fn ids -> Enum.map(ids, &Users.get_name/1) end\n    )","ref":"LiveFilter.UIState.html#extract_filter_value/3-examples"},{"type":"type","title":"LiveFilter.UIState.extractor_fun/0","doc":"","ref":"LiveFilter.UIState.html#t:extractor_fun/0"},{"type":"function","title":"LiveFilter.UIState.filters_to_ui_state/2","doc":"Extract all UI-relevant values from filters at once.\nReturns a map with common UI state fields populated.","ref":"LiveFilter.UIState.html#filters_to_ui_state/2"},{"type":"function","title":"Options - LiveFilter.UIState.filters_to_ui_state/2","doc":"* `:search_field` - Field name for search (default: `:_search`)\n  * `:fields` - List of {field, type} tuples to extract\n  * `:transform` - Map of field -> transform function","ref":"LiveFilter.UIState.html#filters_to_ui_state/2-options"},{"type":"function","title":"LiveFilter.UIState.merge_ui_with_filters/3","doc":"Convert a FilterGroup with UI state to filters.\nHelper for when you have both a FilterGroup and separate UI state.","ref":"LiveFilter.UIState.html#merge_ui_with_filters/3"},{"type":"type","title":"LiveFilter.UIState.ui_state/0","doc":"","ref":"LiveFilter.UIState.html#t:ui_state/0"},{"type":"function","title":"LiveFilter.UIState.ui_to_filters/2","doc":"Convert UI state to a list of filters.","ref":"LiveFilter.UIState.html#ui_to_filters/2"},{"type":"function","title":"Options - LiveFilter.UIState.ui_to_filters/2","doc":"* `:converter` - Custom converter function (default: `default_converter/2`)\n  * `:fields` - Field configuration for the converter\n  * Any other options are passed to the converter","ref":"LiveFilter.UIState.html#ui_to_filters/2-options"},{"type":"function","title":"Examples - LiveFilter.UIState.ui_to_filters/2","doc":"# Using default converter\n    ui_state = %{\n      search_query: \"urgent\",\n      selected_values: %{status: [:pending, :in_progress]},\n      date_ranges: %{due_date: {~D[2025-01-01], ~D[2025-01-31]}}\n    }\n\n    filters = LiveFilter.UIState.ui_to_filters(ui_state,\n      fields: [\n        {:status, :enum, default_op: :in},\n        {:due_date, :date, default_op: :between}\n      ]\n    )","ref":"LiveFilter.UIState.html#ui_to_filters/2-examples"},{"type":"module","title":"LiveFilter.UrlSerializer","doc":"Handles serialization and deserialization of filter configurations to/from URL parameters.\n\nUses a flat parameter structure similar to Backpex:\n- Simple filters: filters[field_name]=value\n- Range filters: filters[field_name][start]=X&filters[field_name][end]=Y\n- Array filters: filters[field_name][]=value1&filters[field_name][]=value2\n- Operators stored separately: filters[field_name][operator]=contains\n\nSort parameters:\n- Single sort: sort[field]=due_date&sort[direction]=desc\n- Multiple sorts: sort[0][field]=priority&sort[0][direction]=desc&sort[1][field]=due_date","ref":"LiveFilter.UrlSerializer.html"},{"type":"function","title":"LiveFilter.UrlSerializer.from_params/1","doc":"Extracts filter group from URL parameters.","ref":"LiveFilter.UrlSerializer.html#from_params/1"},{"type":"function","title":"LiveFilter.UrlSerializer.pagination_from_params/1","doc":"Extracts pagination configuration from URL parameters.\n\nReturns a map with :page and :per_page keys, or defaults.","ref":"LiveFilter.UrlSerializer.html#pagination_from_params/1"},{"type":"function","title":"LiveFilter.UrlSerializer.sorts_from_params/1","doc":"Extracts sort configuration from URL parameters.\n\nReturns a list of Sort structs or nil if no sort params.","ref":"LiveFilter.UrlSerializer.html#sorts_from_params/1"},{"type":"function","title":"LiveFilter.UrlSerializer.update_pagination_params/2","doc":"Updates URL parameters with pagination values.","ref":"LiveFilter.UrlSerializer.html#update_pagination_params/2"},{"type":"function","title":"LiveFilter.UrlSerializer.update_params/2","doc":"Updates URL parameters with filter values.","ref":"LiveFilter.UrlSerializer.html#update_params/2"},{"type":"function","title":"LiveFilter.UrlSerializer.update_params/3","doc":"Updates URL parameters with filter and sort values.","ref":"LiveFilter.UrlSerializer.html#update_params/3"},{"type":"function","title":"LiveFilter.UrlSerializer.update_params/4","doc":"Updates URL parameters with filter, sort, and pagination values.","ref":"LiveFilter.UrlSerializer.html#update_params/4"},{"type":"function","title":"LiveFilter.UrlSerializer.update_sort_params/2","doc":"Updates URL parameters with sort values only.","ref":"LiveFilter.UrlSerializer.html#update_sort_params/2"},{"type":"module","title":"LiveFilter.UrlUtils","doc":"Utilities for handling URL parameter encoding and decoding.\n\nProvides functions to flatten nested parameter structures for proper URL encoding\nand other URL-related utilities for LiveFilter.","ref":"LiveFilter.UrlUtils.html"},{"type":"function","title":"LiveFilter.UrlUtils.flatten_and_encode_params/1","doc":"Flattens nested parameter maps into a flat structure suitable for URL encoding.\n\nConverts nested maps like `%{\"filters\" => %{\"status\" => %{\"operator\" => \"in\"}}}`\ninto flat parameters like `[{\"filters[status][operator]\", \"in\"}]`.","ref":"LiveFilter.UrlUtils.html#flatten_and_encode_params/1"},{"type":"function","title":"Examples - LiveFilter.UrlUtils.flatten_and_encode_params/1","doc":"iex> params = %{\"filters\" => %{\"status\" => %{\"operator\" => \"in\", \"values\" => [\"pending\"]}}}\n    iex> LiveFilter.UrlUtils.flatten_and_encode_params(params)\n    \"filters%5Bstatus%5D%5Boperator%5D=in&filters%5Bstatus%5D%5Bvalues%5D%5B0%5D=pending\"","ref":"LiveFilter.UrlUtils.html#flatten_and_encode_params/1-examples"},{"type":"function","title":"LiveFilter.UrlUtils.flatten_params/2","doc":"Flattens nested parameters into a list of key-value tuples.","ref":"LiveFilter.UrlUtils.html#flatten_params/2"},{"type":"function","title":"Examples - LiveFilter.UrlUtils.flatten_params/2","doc":"iex> params = %{\"user\" => %{\"name\" => \"John\", \"tags\" => [\"admin\", \"user\"]}}\n    iex> LiveFilter.UrlUtils.flatten_params(params)\n    [{\"user[name]\", \"John\"}, {\"user[tags][0]\", \"admin\"}, {\"user[tags][1]\", \"user\"}]","ref":"LiveFilter.UrlUtils.html#flatten_params/2-examples"},{"type":"task","title":"mix live_filter.install.assets","doc":"Installs LiveFilter JavaScript assets to your Phoenix application.\n\nThis task copies JavaScript hooks from the LiveFilter library to your\napplication's assets directory so they can be imported and used with\nPhoenix LiveView.","ref":"Mix.Tasks.LiveFilter.Install.Assets.html"},{"type":"task","title":"Usage - mix live_filter.install.assets","doc":"$ mix live_filter.install.assets\n\nThis will copy the LiveFilter JavaScript hooks to:\n\n    assets/js/hooks/live_filter/","ref":"Mix.Tasks.LiveFilter.Install.Assets.html#module-usage"},{"type":"task","title":"Integration - mix live_filter.install.assets","doc":"After running this task, add the unified hook to your app.js:\n\n    import LiveFilter from \"./hooks/live_filter/live_filter\"\n    \n    let liveSocket = new LiveSocket(\"/live\", Socket, {\n      hooks: { LiveFilter }\n    })","ref":"Mix.Tasks.LiveFilter.Install.Assets.html#module-integration"},{"type":"task","title":"Included Files - mix live_filter.install.assets","doc":"- `live_filter.js` - Unified hook for all LiveFilter components (recommended)\n- `date_calendar_position.js` - Legacy hook for backward compatibility\n- `index.js` - Module exports for both approaches","ref":"Mix.Tasks.LiveFilter.Install.Assets.html#module-included-files"},{"type":"task","title":"Migration from Legacy Hooks - mix live_filter.install.assets","doc":"If you're upgrading from an older version that used individual hooks:\n\n1. Replace `import DateCalendarPosition from \"./hooks/live_filter/date_calendar_position\"`\n2. With `import LiveFilter from \"./hooks/live_filter/live_filter\"`\n3. Update hooks: `{ DateCalendarPosition }` to `{ LiveFilter }`\n\nThe old hooks remain available for backward compatibility.","ref":"Mix.Tasks.LiveFilter.Install.Assets.html#module-migration-from-legacy-hooks"},{"type":"extras","title":"LiveFilter","doc":"# LiveFilter\n\nA flexible and composable filtering library for Phoenix LiveView applications, inspired by Linear, Notion and Airtable. Provides filtering, sorting, and pagination with clean URL state management and a modern UI built on [SaladUI](https://salad-storybook.fly.dev/) and inspired by [shadcn/ui data tables](https://tablecn.com/).\n\n[![Hex.pm](https://img.shields.io/hexpm/v/livefilter.svg)](https://hex.pm/packages/livefilter)\n[![Documentation](https://img.shields.io/badge/documentation-hexdocs-blue.svg)](https://hexdocs.pm/livefilter)\n\n**Demo**: [https://livefilter.fly.dev](https://livefilter.fly.dev/) - source: [https://github.com/cpursley/livefilter-demo](https://github.com/cpursley/livefilter-demo)","ref":"readme.html"},{"type":"extras","title":"Features - LiveFilter","doc":"- Filter operators for all data types (string, numeric, boolean, date, enum, array)\n- URL state management with shareable filter links\n- Autogeneration of database-efficient queries\n- Sorting and column management\n- Components built on shadcn-inspired [SaladUI](https://salad-storybook.fly.dev/)","ref":"readme.html#features"},{"type":"extras","title":"Installation - LiveFilter","doc":"Add to your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:livefilter, \"~> 0.1.0\"}\n  ]\nend\n```\n\nInstall JavaScript assets:\n\n```bash\nmix live_filter.install.assets\n```\n\nAdd to your `app.js`:\n\n```javascript\nimport LiveFilter from \"./hooks/live_filter/live_filter\"\n\nlet liveSocket = new LiveSocket(\"/live\", Socket, {\n  hooks: { LiveFilter }\n})\n```","ref":"readme.html#installation"},{"type":"extras","title":"Basic Usage - LiveFilter","doc":"","ref":"readme.html#basic-usage"},{"type":"extras","title":"1. Minimal Setup - LiveFilter","doc":"```elixir\ndefmodule MyAppWeb.ProductLive do\n  use MyAppWeb, :live_view\n  alias LiveFilter.{Filter, FilterGroup, QueryBuilder}\n\n  def mount(_params, _session, socket) do\n    {:ok, assign(socket, :products, load_products())}\n  end\n\n  defp load_products(filter_group \\\\ %FilterGroup{}) do\n    from(p in Product)\n    |> QueryBuilder.build_query(filter_group)\n    |> Repo.all()\n  end\nend\n```","ref":"readme.html#1-minimal-setup"},{"type":"extras","title":"2. URL-Persisted Filters - LiveFilter","doc":"```elixir\ndefmodule MyAppWeb.ProductLive do\n  use MyAppWeb, :live_view\n  use LiveFilter.Mountable  # Adds filter helpers\n\n  def mount(_params, _session, socket) do\n    socket = mount_filters(socket)\n    {:ok, assign(socket, :products, [])}\n  end\n\n  def handle_params(params, _url, socket) do\n    socket = handle_filter_params(socket, params)\n    {:noreply, assign(socket, :products, load_products(socket.assigns.filter_group))}\n  end\n\n  defp load_products(filter_group) do\n    from(p in Product)\n    |> QueryBuilder.build_query(filter_group)\n    |> Repo.all()\n  end\nend\n```","ref":"readme.html#2-url-persisted-filters"},{"type":"extras","title":"Advanced Example - LiveFilter","doc":"Here's a complete implementation with UI components (similar to our [demo app](https://livefilter.fly.dev/)):\n\n```elixir\ndefmodule MyAppWeb.TodoLive do\n  use MyAppWeb, :live_view\n  use LiveFilter.Mountable\n\n  def mount(_params, _session, socket) do\n    socket = \n      socket\n      |> mount_filters(registry: field_registry())\n      |> assign(:todos, [])\n    \n    {:ok, socket}\n  end\n\n  def handle_params(params, _url, socket) do\n    socket = handle_filter_params(socket, params)\n    {:noreply, assign(socket, :todos, load_todos(socket.assigns.filter_group))}\n  end\n\n  def render(assigns) do\n    ~H\"\"\"\n     \n      <!-- Search and quick filters -->\n       \n         \n           \n         \n        \n        <.live_component \n          module={LiveFilter.Components.SearchSelect}\n          id=\"status-filter\"\n          field={:status}\n          value={get_filter_value(@filter_group, :status)}\n          options={[\n            {\"pending\", \"Pending\"},\n            {\"in_progress\", \"In Progress\"}, \n            {\"completed\", \"Completed\"}\n          ]}\n        />\n       \n\n      <!-- Advanced filter builder -->\n      <.live_component\n        module={LiveFilter.Components.FilterBuilder}\n        id=\"filter-builder\"\n        filter_group={@filter_group}\n        field_options={field_options()}\n      />\n\n      <!-- Results table -->\n       \n         \n           \n            <.live_component \n              module={LiveFilter.Components.SortableHeader}\n              id=\"sort-title\"\n              field={:title}\n              label=\"Title\"\n              current_sort={@current_sort}\n            />\n             Status \n             Due Date \n           \n         \n         \n           \n             <%= todo.title %> \n             <%= todo.status %> \n             <%= todo.due_date %> \n           \n         \n       \n     \n    \"\"\"\n  end\n\n  def handle_event(\"search\", %{\"q\" => query}, socket) do\n    filter = %Filter{field: :title, operator: :contains, value: query, type: :string}\n    filter_group = %FilterGroup{filters: [filter]}\n    socket = apply_filters_and_reload(socket, filter_group)\n    {:noreply, assign(socket, :search, query)}\n  end\n\n  defp field_registry do\n    LiveFilter.FieldRegistry.new()\n    |> LiveFilter.FieldRegistry.put(:title, :string, \"Title\")\n    |> LiveFilter.FieldRegistry.put(:status, :enum, \"Status\", \n         options: [{\"pending\", \"Pending\"}, {\"in_progress\", \"In Progress\"}, {\"completed\", \"Completed\"}])\n    |> LiveFilter.FieldRegistry.put(:due_date, :date, \"Due Date\")\n    |> LiveFilter.FieldRegistry.put(:tags, :array, \"Tags\")\n  end\n\n  defp load_todos(filter_group) do\n    from(t in Todo)\n    |> QueryBuilder.build_query(filter_group)\n    |> QueryBuilder.apply_sort(socket.assigns.current_sort)\n    |> Repo.all()\n  end\nend\n```","ref":"readme.html#advanced-example"},{"type":"extras","title":"Documentation - LiveFilter","doc":"- [API Documentation](https://hexdocs.pm/livefilter)\n- [Demo Application](https://github.com/cpursley/livefilter-demo)","ref":"readme.html#documentation"},{"type":"extras","title":"License - LiveFilter","doc":"MIT","ref":"readme.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}